package ShapeQuery;
// BandJoinRule.java: Rule for performing band join, to grab first two
//                    edges in one pass through the edge list.
//
// Note: this is not a robust implementation of band join.  The
// algorithm is implemented for distances only, and there are certain
// bits of "glue" that are glossed over or hard coded and therefore
// won't work in many cases.
//
// Also, this implementation works only to join a PointPairIndex to
// itself to get an initial result.  In theory it should be able to
// join two separate QuerySolutionSets together.  In practice this
// approach is almost definitely slower and therefore I'm not sure
// it's worth implementing.
//
// See explanation/example in PointPairIndex.java

import java.util.*;  // for vector
import java.io.*;    // for reading files
import java.net.*;

// for drawing
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.Line2D;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;
import java.awt.event.*;


class BandJoinRule extends ShapeQueryRule
{
    // the names of the points to use when performing this join
    private String pointNames[][];

    // the constraint to use
    Constraint constraint;

    // make a new band join rule
    BandJoinRule(Constraint _constraint)
    {
	constraint = _constraint.copy();
    }

    // print out this rule
    void print()
    {
	OutputWriter.println("> band join rule for constraint:");
	constraint.print();
    }

    String getDescription() {
        String s = "BandJoinRule";
        return s;
        
    }    

    // cost estimation: it may be the case that a band join is always
    // cheapest, in which case cost estimation is unimportant.
    float estimateCost(PointUniverse universe, float rows)
    {
	return 1.0f;
    }

    float estimateReturnedRowMultiplier(PointUniverse universe)
    {
	return 1.0f;
    }

    // run band join
    QuerySolutionSet apply(QuerySolutionSet solutions,
			   PointSet pointSet)

    {
	OutputWriter.printDebugDetail("  applying band join rule");
	// for point: "+myNewPointName);
	
	// 'solutions' must be empty for band join to work
	if (solutions.size()!=0)
	{
	    OutputWriter.println("Error: solution set is nonempty");
	    return null;
	}

        IndexType type = null;
	switch(constraint.myRuleType)
	{
	    case Constraint.RULE_TYPE_DISTANCE:
	    {
                type = IndexType.ID_DISTANCE;
	    }
	    case Constraint.RULE_TYPE_ANGLE:
	    {
                type = IndexType.ID_ANGLE;
	    }
	}
        return (pointSet.getIndex(type)).bandJoin(constraint, solutions.getEdgesToDraw());

    }
}


