/*
 * CssKey.java
 *
 * Created on October 4, 2005, 2:15 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package ShapeQuery;

/**
 * This class stores a Css Tree Key. The Index would be sorted on the basis of FIRST, SECOND
 * keys. 
 *
 * @author Pranay Jain
 */
public class CssKey implements Comparable {
    
    private float key1;
    private float key2;

    
    /** Creates a new instance of CssNode */
    public CssKey(float key1, float key2) {
        
        this.key1 = key1;
        this.key2 = key2;
        
    }
    
    
   // to determine ordering in array, so we can sort
    public int compareTo(Object o)
    {
	CssKey key = (CssKey)o;

        // iterate through each layer of index.  any time we see a
	// difference, stop looking and return the value according to
	// that difference.  if we don't see any differences, we'll
	// return 0 in the end.
	
        if(key1 < key.key1)
            return -1;
        if(key1 > key.key1)
            return 1;
        
        if(key2 < key.key2)
            return -1;
        if(key2 > key.key2)
            return 1;

      	// no difference!
        return 0;
        
    }
    
        /**
     * Compares this node to values in a 2D array. In the first dimension,
     * the array must be the same length as indexOrder. In the second dimension,
     * the array must be of length 2. The comparison iterates through each 
     * layer i of the index, and returns a negative value if the value of this 
     * node is less than range[i][0], and a positive value if the value of this 
     * node is greater than range[i][1]. Returns 0 if this node's values fall 
     * between the values of range[i][0] and range[i][1] for all i.
     *
     * To do equality tests simply set range[i][0] = range[i][1] = value.
     *
     * This method allows for easy searches through the index.
     *
     * @param range A 2D array as described above.
     * @return -1 if this node is less than range, 1 if this node is greater than 
     * range. 0 otherwise.
     *
     */
    public int compareTo(float[][] range) throws IllegalArgumentException
    {
        //check that range has the right properties. dimension 1 must be as long as 2 (Keys)
        //dimension 2 must be 2.
        if (range.length != 2 || range[0].length != 2) {
            OutputWriter.printerror("2D array 'range' is not correctly sized.");
            throw new IllegalArgumentException("2D array 'range' is not correctly sized.");
        }

	// iterate through each layer of keys.  any time we see a
	// difference, stop looking and return the value according to
	// that difference.  if we don't see any differences, we'll
	// return 0 in the end.
        if(key1 < range[0][0])
            return -1;
        if(key1 > range[0][1])
            return 1;
        
        if(key2 < range[1][0])
            return -1;
        if(key2 > range[1][1])
            return 1;

	// no difference!
	return 0;
    }

    
    
    /**
     * Returns a string containing the details of this key for testing.
     */
    public String getDescription() {
        String s = "KEY(";
        s += key1 + "," + key2;
        s += ") ";
        
        return s;
    }
}
