/*
 * Database.java
 *
 * Created on February 3, 2006, 6:11 PM
 *
 * This class was written purely for storing information in the database for statistical analysis being
 * carried out across churches.
 */

package ShapeQuery;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.util.Vector;

/**
 *
 * @author Pranay Jain
 */
public class Database {
    
    String fileName;
    private String driver = "sun.jdbc.odbc.JdbcOdbcDriver";
    private String url = "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=";
    private Connection con = null;
  
    
    /** Creates a new instance of Database */
    public Database(String path) {
        try {
            fileName = path + "ShapeQuery.mdb";
            //System.out.println(fileName);
            url = url + fileName + ";DriverID=22;READONLY=true}"; 
            Class.forName( driver );
            con = DriverManager.getConnection( url ,"",""); 
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void close() {
        try {
            if(QueryBean.STATISTICAL_ANALAYSIS)
                con.close();
        } catch (Exception e) {
            e.printStackTrace();
        }    
    }
    
    public void deletePointSet (String churchName) {
	try {
            String sql = "DELETE FROM pointSet WHERE churchName = ?";
	    PreparedStatement psmt = con.prepareStatement(sql);

            psmt.setString(1, churchName);
            
            psmt.executeUpdate();
            
        } catch (Exception e) {
            e.printStackTrace();
        }  
        
    }

    
    public void loadPointSet (String churchName, String pointID, Vector<String> labels, float crdX, float crdY) {
	try {
            String sql = "INSERT INTO pointSet(churchName, pointID, label, crdX, crdY) VALUES(?,?,?,?,?)";
	    PreparedStatement psmt = con.prepareStatement(sql);

            String label = labels.get(0);
            psmt.setString(1, churchName);
            psmt.setString(2, pointID);
            psmt.setString(3, label);
            psmt.setFloat(4, crdX);
            psmt.setFloat(5, crdY);
            
            psmt.executeUpdate();
            
        } catch (Exception e) {
            e.printStackTrace();
        }          
    }

    public void deleteResultSet (String churchName, String queryName) {
	try {
            String sql = "DELETE FROM resultSet WHERE churchName=? AND queryName=?";
	    PreparedStatement psmt = con.prepareStatement(sql);
              
            psmt.setString(1, churchName);
            psmt.setString(2, queryName);

            psmt.executeUpdate();            
            
        } catch (Exception e) {
            e.printStackTrace();
        }          
    }

    
    public void loadResultSet (String churchName, String queryName, Integer resultID, String queryPointID, String pointID) {
	try {
            String sql = "INSERT INTO resultSet(churchName, queryName, resultID, queryPointID, pointID) VALUES(?,?,?,?,?)";
	    PreparedStatement psmt = con.prepareStatement(sql);

            psmt.setString(1, churchName);
            psmt.setString(2, queryName);
            psmt.setInt(3, resultID);
            psmt.setString(4, queryPointID);
            psmt.setString(5, pointID);

            psmt.executeUpdate();                       
            
        } catch (Exception e) {
            e.printStackTrace();
        }          
    }
    
    public void setResultSet (String churchName, String queryName, Integer resultID) {
	try {
            String sql = "UPDATE resultSet SET ignoreFlag = 1 WHERE churchName=? AND queryName=? AND resultID=?";
	    PreparedStatement psmt = con.prepareStatement(sql);

            psmt.setString(1, churchName);
            psmt.setString(2, queryName);
            psmt.setInt(3, resultID);

            psmt.executeUpdate();                       
            
        } catch (Exception e) {
            e.printStackTrace();
        }              
    }
    
}
