package ShapeQuery;
// FirstEdgeRule.java: the rule to execute to populate an empty
//                     solution set with all the possible edges in the
//                     point set; this is done at the beginning of
//                     every query (unless a band join is used)

import java.util.*;  // for vector
import java.io.*;    // for reading files
import java.net.*;

// for drawing
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.Line2D;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;
import java.awt.event.*;


class FirstEdgeRule extends ShapeQueryRule
{
    // the names of the two points that will form the first edge in a
    // query
    String myFirstPointName;
    String mySecondPointName;
    
    Constraint con;


    // make a new rule
    FirstEdgeRule(Constraint constraint) 
    {
        myFirstPointName = constraint.myRequiredEdgeStartPointName;
	mySecondPointName = constraint.myRequiredEdgeEndPointName;
        con = constraint.copy();
    }


    // print basic info
    void print()
    {
	OutputWriter.println("> first-edge rule for points: "
			   +myFirstPointName+", "+mySecondPointName);
    }


    String getDescription() {
        String s = "FirstEdgeRule: ";
        s += "(" + myFirstPointName + "," + mySecondPointName + ")";
        return s;
        
    }    
    
    // Cost/rows estimation; given this universe (and this number of
    // input rows), estimate how many rows will be returned and how
    // expensive this rule will be.
    // 
    // Note that since this is always the first rule, these numbers
    // will be the same for all candidate plans, so it doesn't really
    // matter what they are.  I chose these numbers to try to give an
    // answer that makes sense to compare across point sets, but this
    // could probably be improved with a bit more thought.
    float estimateCost(PointUniverse universe, float rows)
    {
	return universe.averageRowCount();
    }

    float estimateReturnedRowMultiplier(PointUniverse universe)
    {
	float rowCount = universe.averageRowCount();

        //reduce the rowcount by the label reduction factor - DJH
        rowCount *= computeLabelReduction(con.getAllLabels(), universe);
        return rowCount;
    }


    // given a solution set (which is ignored), return all the pairs
    // of points in this point set
    QuerySolutionSet apply(QuerySolutionSet<QuerySolution> solutions,
			   PointSet pointSet)
    {
	OutputWriter.printDebugDetail("  applying firstedge rule for points: "
	    +myFirstPointName+", "+mySecondPointName);
	solutions.clear();
	Iterator iter = pointSet.pointPairs.iterator();
	
	int count = 0;
	while (iter.hasNext())
	{
	    PointPair current = (PointPair)(iter.next());

	    QuerySolution solution = new QuerySolution();
	    
	    if (current.firstPoint.labelMatches(con.myRequiredEdgeStartPointLabels) && 
                current.secondPoint.labelMatches(con.myRequiredEdgeEndPointLabels) &&
                solution.addPoint(myFirstPointName, current.firstPoint) &&
		solution.addPoint(mySecondPointName, current.secondPoint))
	    {
		solutions.add(solution);
	    }
	    count++;
	    if (count%1000==0)
		OutputWriter.printDebugDetail(".");
	}
	
	return solutions;
    }
    
    
    boolean edgeToSelf() {
        if (myFirstPointName.equals(mySecondPointName))
            return true;
        return false;
    }
}
