/*
 * IndexTypes.java
 *
 * Created on June 9, 2005, 2:34 PM
 */

package ShapeQuery;

/**
 * This nifty little class is basically just a bunch of tricks
 * to make creating, accessing, and keeping track of indices easier and 
 * cleaner. It mimicks an enumerable constant of the kind available in 
 * languages like Pascal and C++. For a full explanation of the concepts 
 * see http://www.javaworld.com/javaworld/jw-07-1997/jw-07-enumerated.html
 *
 * @author David Hessing
 */
public final class IndexType {
    public static final int FIRSTID  = 0;
    public static final int SECONDID = 1;
    public static final int DISTANCE = 2;
    public static final int ANGLE    = 3;
    
    private String type;
    public final int ord;
    private int[] indexOrder;
    private IndexType prev;
    private IndexType next;
    
    private static int upperBound = 0;
    private static IndexType first = null;
    private static IndexType last = null;
    
    private IndexType(String aType, int[] ordering) {
        this.type = aType;
        this.indexOrder = ordering;
        this.ord = upperBound++;
        if (first == null)
            first = this;
        if (last != null) {
            this.prev = last;
            last.next = this;
        }
        last = this;
    }
    
    public String toString() {return this.type;}
    public static int size() {return upperBound;}
    public static IndexType first() {return first;}
    public static IndexType last() {return last;}
    public IndexType prev() {return this.prev;}
    public IndexType next() {return this.next;}
    public int[] order() {return indexOrder;}
    
    //These static final objects act like enums. They are the list of allowable indices.
    public static final IndexType ID_DISTANCE = new IndexType("ID_Distance   ", new int[] {FIRSTID, DISTANCE});
    public static final IndexType ID_ANGLE = new IndexType("ID_Angle      ", new int[] {FIRSTID, ANGLE});
    public static final IndexType DISTANCE_ANGLE = new IndexType("Distance_Angle", new int[] {DISTANCE, ANGLE});
    public static final IndexType ANGLE_DISTANCE = new IndexType("Angle_Distance", new int[] {ANGLE, DISTANCE});
    
}
