package ShapeQuery;
// OutputWriter.java: handle outputting of informational/debugging
//                    text in a modular way, so that it's easy to
//                    change where this output goes

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.FileWriter;
import java.io.File;

class OutputWriter
{
    //String userDir =  System.getProperty("user.dir") + "\\log\\";
    public static String loglocation = null; 
    public static final int MAX_BYTES = 10000;

    public static boolean PERFORMANCETEST = false;
    public static boolean DEBUG = false;
    public static boolean DEBUGDETAIL = false;
    public static boolean SHOWINTERMEDIATESVG = false;
    public static boolean SHOWERRORS = false;

    public static int errcount;

    private static BufferedWriter output;
    private static FileWriter debugoutput;

    OutputWriter()
    {
        errcount = 0;
    }

    static void setOutput(BufferedWriter writer)
    {
        errcount = 0;
	output = writer;
    }
    
    static void setLogLocation(String path) {
        loglocation = path;
    }

    static void clearOldSVG(String setName) {
         //delete old svg log files for this pointset
        File f = new File(loglocation);
        String old[] = f.list();
        if (old == null) 
            return;
        for (int i = 0; i < old.length; i++) {
            if (old[i].startsWith(setName)) {
                f = new File(loglocation + old[i]);
                f.delete();
            }
        }          
    }
    
    static void printSVGResults(String filename, String ruleDesc, QuerySolutionSet solutions, PointSet ps)
    {
        if (!SHOWINTERMEDIATESVG || loglocation == null)
            return;

        if (!filename.endsWith(".svg")) {
            filename += ".svg";
        }
        try {
            FileWriter fw = new FileWriter(loglocation + filename, false);
            QuerySolutionSet.writeSVGSolutions(ruleDesc,"", "", solutions, ps, fw, 1.0f, 50.0f);
        } 
        catch(IOException ioe) {
            String s = "WRITING TO SVG FILE FAILED: " + ioe.getMessage() + "\n";
            System.out.print(s);
        }
        
    }

    static void print(String s)
    {
        //don't print anything if we aren't in debug mode
        if (!DEBUG)
            return;
        
        //printit(s);
        
        try {
            if (debugoutput==null) {
                debugoutput = new FileWriter(loglocation + "log.txt", false);
            }

            debugoutput.write(s);
            debugoutput.flush();
        } catch(IOException ioe) {
            System.out.print(s + " (WRITING TO LOG FILE FAILED: " + ioe.getMessage() + ")\n");
        }
    }

    static void printstat(String s)
    {
        if (PERFORMANCETEST) {
            DEBUG = true;
            println(s);
            DEBUG = false;
        }    
    }
    
    static void printerror(String s)
    {
        if (SHOWERRORS)
            printit(s);
    }

    private static void printit(String s)
    {
       	if (output==null)
	{
	    System.out.print(s);
	}
	else
	{
            try {
                if (errcount == 0) {output.write("<?xml version=\"1.0\"?>\n<results>"); }
                output.write("<error>" + s.replace("\n", "") + "</error>");
                output.flush();
                errcount++;
            }
            catch (IOException ioe) {
                System.out.print(s + " (WRITING TO SPECIFIED WRITER FAILED: " + ioe.getMessage() + ")\n");
            }
	}

    }
    
    
    static void printDebugDetail(String s)
    {
        if (DEBUGDETAIL) {
            println(s);
        }    
    }    
    
    static void println(String s)
    {
	print(s+"\n");
    }
}
