package ShapeQuery;
// Point.java: class that represents a point in 2d space.

import java.util.*;  // for vector
import java.io.*;    // for reading files

// for drawing
//import javax.swing.*;
//import javax.swing.event.*;
//import java.awt.geom.Line2D;
//import java.awt.*;
//import java.awt.image.*;
//import javax.imageio.*;
//import java.awt.event.*;
import java.util.Vector;

class Point
{
    private static int nextId = 0;  // id of next point we'll create

    private float x, y;             // coordinates of this point

    private int id;                 // id of this point; used for
				    // comparison and natural
				    // ordering; note that if a point
				    // with the same coordinates is
				    // created twice, each will have a
				    // unique id.
    

    //the following variables were added by DJH
    private String externalID; //optional ID of external application. Assumed unique. Output in results if set. 
    private boolean referenceVector; //indicates if this point part of a reference vector. ie, it should be in indices in some capacity, but not in results at all.
    private Vector labels;
    
    // make a new point
    Point(float _x, float _y)
    {
        init(_x, _y, "", new Vector(), false);
    }

    Point (float _x, float _y, String _externalID, Vector _labels, boolean _referenceVector) 
    {
        init(_x, _y, _externalID, _labels, _referenceVector);
    }
    
    Point (float _x, float _y, String _externalID, Vector _labels) 
    {
        init(_x, _y, _externalID, _labels, false);
    }

    Point (float _x, float _y, String _externalID) 
    {
        init(_x, _y, _externalID, new Vector(), false);
    }
    
    private void init(float _x, float _y, String _externalID, Vector _labels, boolean _referenceVector) {
        x = _x;
        y = _y;
        
        id = nextId;
        nextId++;
        
        externalID = _externalID;
        referenceVector = _referenceVector;
        labels = _labels;
        
    }

    /**
     * Checks if this point has some label that matches ONE of the labels in 
     * the passed in vector. If the passed in vector is null or empty, returns true since 
     * by the query construction if no label restrictions are specified then all
     * points qualify.
     *
     * @param someLabels a Vector of labels, specified in a query.
     * @return true if this point has a label that matches one of the labels in someLabels
     */
    public boolean labelMatches(Vector someLabels) {
        if(someLabels == null || someLabels.size() == 0) {
            return true;
        }
        
        if(labels == null || labels.size() == 0) {
            return false;
        }
        
        String l1 = "";
        String l2 = "";
        for(int i = 0; i < labels.size(); i++) {
            l1 = (String)labels.get(i);
            for(int j = 0; j < someLabels.size(); j++) {
                l2 = (String)someLabels.get(j);
                if (l1.equals(l2)) {return true;}
            }
        }
        return false;
    }
    
    // return true if these two points have the same coordinates; note
    // that this is different from testing for point equality because
    // the same point can be created twice, and will then have two
    // distinct instances.
    //
    // this function should probably be used more extensively, and the
    // == testing less
    static boolean isEqual(Point a, Point b)
    {
	if (a.getX() == b.getX() && a.getY() == b.getY())
	    return true;
	return false;
    }


    // accessor functions
    float getX()
    {
	return x;
    }

    float getY()
    {
	return y;
    }

    int getId()
    {
	return id;
    }
    
    
    /**
     * Indicates whether there an external ID was set for this point. 
     */
    public boolean hasExternalID() {
        if (externalID != null && externalID.length() > 0) {
            return true;
        }
        return false;
    }
    
    /**
     * Returns the externalID, or the empty string if none was set.
     */
    public String getExternalID() {
        return externalID;
    }
    
    
    /**
     * Returns a string containing the details of this point for testing.
     */
    public String getDescription() {
       String s = "(" + x + "," + y + "): ";
       s += externalID + " " + referenceVector + " [";
       for (int i = 0; i < labels.size(); i++) {
           s += ((String)labels.get(i)) + ",";
       }
       //s = s.substring(0, s.lastIndexOf(",")) + "]";
       s += "]";
       return s;
    }
    
}
