package ShapeQuery;
// PointPair.java: class representing a pair of points (which may form
//                 an edge of a shape)

import java.util.*;  // for vector
import java.io.*;    // for reading files

// for drawing
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.Line2D;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;
import java.awt.event.*;

class PointPair
{
    // a pointpair is composed of points
    public Point firstPoint, secondPoint;

    // the distance and angle associated with this pointpair
    public float distance;    // the distance between these two points
    public float angle;       // angle with the x axis
    
    // make a new point pair from two points
    PointPair(Point first, Point second)
    {
	if (first==null || second==null)
	{
	    OutputWriter.println("error: null point specified");
	    return;
	}

	// store what points we have
	firstPoint = first;
	secondPoint = second;


	// grab point locations to compute distance and angle (so that
	// if we need to access it many times, we don't need to
	// compute it many times)
	float firstX = first.getX();
	float firstY = first.getY();

	float secondX = second.getX();
	float secondY = second.getY();

	distance = (float) Math.sqrt(
	    (firstX-secondX)*(firstX-secondX)+
	    (firstY-secondY)*(firstY-secondY));
        
	angle = 
	    ( (float)Math.atan2( firstX-secondX, firstY-secondY))
	    *180f/((float)Math.PI);
    }

    // points have ids to create a concept of a "natural order" for
    // sorting, matching.  (pretty artificial actually)
    public int getFirstId()
    {
	return firstPoint.getId();
    }

    public int getSecondId()
    {
	return secondPoint.getId();
    }


    // print out this point pair in a reasonable way
    public void print()
    {
	OutputWriter.println("("
			     +firstPoint.getX()
			     +","
			     +firstPoint.getY()
			     +"),("
			     +secondPoint.getX()
			     +","
			     +secondPoint.getY()
			     +") ["
			     +distance+","+angle+"]");
    }
}
