/*
 * QueryBean.java
 *
 * Created on March 20, 2005, 1:14 PM
 * Provides a single class for the JSP pages to interface with.
 */

package ShapeQuery;

import java.beans.*;
import java.io.Serializable;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Calendar;

/**
 * Provides a single class for the JSP pages to interface with.
 * @author David Hessing
 */

public class QueryBean extends Object implements Serializable {
    
    /* This is Flag which was setup only to facilitate staticstical analysis of churches
       (carried out at the end of the project) and store information in the database. 
       It should always be false before shipping the product. -- Pranay
    */
    public static final boolean STATISTICAL_ANALAYSIS = false;
    public static final boolean SERVER = true;
    public static Database db = null;
    
    private String pointsetURL;
    private String queryURL;
    private String path; //the real path on the server where the JSP pages are running.
    private ShapeQueryExecuter sqe;
    private HashMap<String, PointSet> preloadedSets; //cached pointsets
    private long lastused;                           //for cache life
    
    public QueryBean() { 
        sqe = new ShapeQueryExecuter();
        preloadedSets = new HashMap<String, PointSet>();
        lastused = Calendar.getInstance().getTimeInMillis();
    }   
    
    public String getPointsetURL() {
        return pointsetURL;
    }
    
    public void setPointsetURL(String value) {
        pointsetURL = value;
        if(STATISTICAL_ANALAYSIS)
            db = new Database(path);
    }

    public String getQueryURL() {
        return queryURL;
    }
    
    public void setQueryURL(String value) {
        queryURL = value;
    }

    public void setPath(String value) {
        path = value;
        if(QueryBean.SERVER)
            path += "/bourbonnjsp/";
        else
            path += "\\web\\";
    }
    
    public long getTimeNotUsed()
    {
        long now = Calendar.getInstance().getTimeInMillis();
        return (now - lastused);
    }

    /**
     * Constructs URL objects from the queryURL and pointsetURL parameter. Retrieves the XML from these
     * locations, parses it, and executes the query. Writes the results out to the specified BufferedWriter,
     * which should be the special JSP "out" variable.
     *
     * @param A writer to write the results or error to. Should be the JSP "out" variable.
     */
    public void executeQuery(BufferedWriter writer) {
        //update lastused time to extend this Bean's cache life
        lastused = Calendar.getInstance().getTimeInMillis();
        
        //set so that error messages are written out to the web
        OutputWriter.setOutput(writer);
        //set the log location
        if(QueryBean.SERVER)   
            OutputWriter.setLogLocation(path + "log/");
        else
            OutputWriter.setLogLocation(path + "log\\");
        
        //convert to URLs
        URL query = null;
        URL points = null;
        try { 
            query = new URL(queryURL);
            points = new URL(pointsetURL);
        }
        catch (MalformedURLException mue) {
            OutputWriter.printerror("QueryBean - One or both URLs are bad: " + mue.getMessage() + "\n");
            return;
        }
        
        //try to parse the XML files
        sqe.setQuery(QueryLoader.loadXMLQuery(query));
        sqe.setUniverse(QueryLoader.loadXMLDataSets(points, preloadedSets));
        
        //run the query
        int results = sqe.executeQuery();
        if (results < 0) {
            OutputWriter.printerror("QueryBean - Query failed.\n");
        }
        
        //check if there were errors if so write out final root tag (the error handling for the xml is disorganized).
        try {
            if (OutputWriter.errcount > 0) {
                writer.write("</results>");
                writer.flush();
            }
        } catch (IOException ioe) {}
    }
    
    
    public void writeXMLResults(BufferedWriter writer) {
        sqe.writeXMLresults((java.io.Writer)writer);
    }
    
    public void writeSVGResults() {
        sqe.writeSVGresults(path);
    }
    
    
}
