/*
 * QueryProfiler.java
 *
 * Created on February 23, 2005, 10:22 AM
 * Provides functionality for testing query execution and runtime.
 */
package ShapeQuery;


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * Provides functionality for testing query execution and runtime.
 * @author David Hessing
 */
public class QueryProfiler {

    public enum testtype {RunOnEach, RunOnAll, RunConsec;}

    private static final String TESTS[] = {"config_test_setnumber.txt", 
                                    "config_test_setsize.txt"};
    //and a parallel array of testtypes
    private static final testtype TESTTYPES[] = {testtype.RunConsec, testtype.RunOnEach};

    private static final int REPEATTESTS = 10;

    
    /** Creates a new instance of QueryProfiler */
    public QueryProfiler() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        QueryLoader ql = new QueryLoader(TESTS);
        
        //test xml input
        testXMLin();

        //test xml output
        //testXMLout(ql, 0);
         
         //run the specified tests
         //for (int i = 0; i < TESTS.length; i++) {
         //   switch (TESTTYPES[i]) {
         //       case RunOnEach:
         //           testOnEach(ql, i);
         //           break;
         //       case RunOnAll:
                    
         //           break;
         //       case RunConsec:
                    //testConsec(ql, i);
         //           break;
         //   }
         //}
        
    }
    
    /**
     * Tests parsing of XML input files.
     */
    private static void testXMLin() {
        try {
            //FileReader pnts = new FileReader("fakepointsets1.xml");
            URL pnts = new URL("http://localhost:8084/ShapeQueryWeb/points/autrybessonIDs.xml");
            QueryLoader.loadXMLDataSets(pnts);
            FileReader qry = new FileReader("queries/square.xml");
            QueryLoader.loadXMLQuery(qry);
        }
        catch (FileNotFoundException fnfe) {
            OutputWriter.printerror("File not found: " + fnfe.getMessage());
        }
        catch (MalformedURLException mue) {
            OutputWriter.printerror("Bad URL: " + mue.getMessage());
        }
    }

    /**
     * Tests writing of XML output.
     */
    private static void testXMLout(QueryLoader ql, int index) {
        DataSet ds[] = ql.getDataSets(index);
        ShapeQuery query = ql.loadQuery(ql.getQueryFileNames(index)[0],  ql.getTolerance(index));
        PointUniverse u = ql.loadDataSets(ds, 0, 3);
        
        ShapeQueryExecuter sqe = new ShapeQueryExecuter();
        sqe.setQuery(query);
        sqe.setUniverse(u);
        int results = sqe.executeQuery();
        
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter("testresults.xml"));
            sqe.writeXMLresults(bw);

        } catch (IOException ioe) {
            OutputWriter.printerror("\nFailed to write results: " + ioe.getMessage() + ".\n");
        }
        
    }
    
    
     /**
      * Using the pointsets and query preloaded in a QueryLoader object, executes the query
      * on EACH pointset individually, testing execution time and memory usuage, and writes out the results.
      * 
      * @param ql A QueryLoader object that has loaded pointsets and query from config files.
      * @param index Integer specifying which config file loaded by the QueryLoader to use.
      */
    private static void testOnEach(QueryLoader ql, int index) {
        OutputWriter.printstat("\n\nTESTING ON EACH DATASET - " + TESTS[index] + " TOL: " + ql.getTolerance(index) + "\n");
        OutputWriter.printstat("    Query is run " + REPEATTESTS + " times on each dataset and average time is reported.\n");
        OutputWriter.printstat("    Columns: DATASET, #RESULTS, PLANTIME, QUERYTIME, MB USED\n\n");
        long timer[] = new long[2]; timer[0] = 0; timer[1] = 0;
        long planTot = 0;
        long queryTot = 0;
        float resultCount = 0;
        double mem = 0;
        
        PointUniverse u = null;
        //get the query. for tests we assume only one query was specified in the config file.
        ShapeQuery query = null;
        //get the datasets
        DataSet ds[] = ql.getDataSets(index);
        
        //run the query on EACH dataset, and average the time before reporting.
        for (int i = 0; i < ds.length; i++) {
            //reset the timers
            planTot = 0;
            queryTot = 0;
            resultCount = 0; //to make sure we are consistently returning the same number of results
            //run query the specified number of times
            for (int j = 0; j < REPEATTESTS; j++) {
                System.gc();
                query = ql.loadQuery(ql.getQueryFileNames(index)[0],  ql.getTolerance(index));
                //get the universe with these datasets
                u = ql.loadDataSets(ds, i, 1);
                resultCount += query.execute(u, true, timer);
                planTot += timer[0];
                queryTot += timer[1];
                //write the results of this run
                //writeResults("results.txt", u, j+1);
            }
            
            mem = ((double)(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())) / 1048576;
            OutputWriter.printstat(ds[i].visibleName + ", " + resultCount/((float)REPEATTESTS) + ", " 
                    + planTot / REPEATTESTS + ", " + queryTot / REPEATTESTS + ", " 
                    + mem + "\n");
        }
        
    }
    
     /**
      * Using the pointsets and query preloaded in a QueryLoader object, executes the query
      * on an additional pointset at each iteration, testing execution time and memory usuage, and writes out the results.
      * 
      * @param ql A QueryLoader object that has loaded pointsets and query from config files.
      * @param index Integer specifying which config file loaded by the QueryLoader to use.
      */
    private static void testConsec(QueryLoader ql, int index) {
        OutputWriter.printstat("\n\nTESTING ON ADDITIONAL DATASET EACH TIME - " + TESTS[index] + " TOL: " + ql.getTolerance(index) + "\n");
        OutputWriter.printstat("    Query is run " + REPEATTESTS + " times on subset of datasets and average time is reported.\n");
        OutputWriter.printstat("    Columns: #DATASETS, #RESULTS, PLANTIME, QUERYTIME, MEMORY USED\n\n");
        long timer[] = new long[2]; timer[0] = 0; timer[1] = 0;
        long planTot = 0;
        long queryTot = 0;
        float resultCount = 0;
        double mem = 0;
        
        PointUniverse u = null;
        //get the query. for tests we assume only one query was specified in the config file.
        ShapeQuery query = null;
        //get the datasets
        DataSet ds[] = ql.getDataSets(index);
        
        //run the query on an addition dataset each time, and average the time before reporting.
        for (int i = 0; i < ds.length; i++) {
//        for (int i = 3; i < 6; i++) {
            //reset the timers
            planTot = 0;
            queryTot = 0;
            resultCount = 0; //to make sure we are consistently returning the same number of results
            //run query the specified number of times
            for (int j = 0; j < REPEATTESTS; j++) {
                System.gc();
                query = ql.loadQuery(ql.getQueryFileNames(index)[0],  ql.getTolerance(index));
                //get the universe with these datasets
                u = ql.loadDataSets(ds, 0, i+1);
                resultCount += (float) query.execute(u, true, timer);
                planTot += timer[0];
                queryTot += timer[1];
            }

            mem = ((double)(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())) / 1048576;
            OutputWriter.printstat((i+1) + ", " + resultCount/((float)REPEATTESTS) + ", " 
                    + planTot / REPEATTESTS + ", " + queryTot / REPEATTESTS + ", " 
                    + mem + "\n");
        }
        
        
    }
    
    
    
    
    
    private static boolean writeResults(String filename, PointUniverse universe, int testRun) {
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(filename, true));
            return writeResults(bw, universe, testRun);

        } catch (IOException ioe) {
            OutputWriter.printerror("\nFailed to write results: " + ioe.getMessage() + ".\n");
            return false;
        }
        
    }
    
    /**
     * Writes the results of the most recently executed query out as easily readable lines to BufferedWriter.
     *
     * @param writer Where to write the results to. 
     * @returns True on successful write. False otherwise.
     */
    private static boolean writeResults(BufferedWriter writer, PointUniverse universe, int testRun) {
        int countcheck = 0;
        int numDatasets = universe.getPointSetCount();
        
        try {
            writer.write("RUN " + testRun + ":\n");
            
            for(int i = 0; i < numDatasets; i++) {
                writeResultSet(writer, universe.getPointSetAt(i));
            }
            writer.write("\n");
            writer.flush();
            writer.close();
        }
        catch (IOException ioe) {
            OutputWriter.printerror("ShapeQueryExecuter: Error writing to BufferedWriter: " + ioe.getMessage() + ".\n");
            return false;
        }
  
        return true;
    }

    
     
    /**
     * Writes out the results of this run.
     * @param writer Where to write the to.
     * @param ps A PointSet with an existing QuerySolutionSet.
     */
    private static void writeResultSet(BufferedWriter writer, PointSet ps) throws IOException {
        writer.write(" POINTSET:"+ ps.getVisibleName() + "\n");
        QuerySolutionSet qse = ps.getSolutionSet();
        Iterator iter = qse.listIterator();
        
        QuerySolution qs = null;
        Iterator piter = null;
        Point p = null;
        
        while (iter.hasNext()) {
            qs = (QuerySolution)iter.next();
            piter = (qs.getPointNames()).keySet().iterator();

            writer.write(" <");
            while (piter.hasNext()) {
                p = (Point) qs.getPoint((String)piter.next());
                writer.write("(" + p.getX() + ",");
                writer.write(p.getY() + ")");
            }
            writer.write(">");
        }
        writer.write("\n");
        
        
    }
    
    
    
    
    
    
    
    
    
    
    
}
