package ShapeQuery;
// QuerySolutionSet.java: A list of solutions resulting from running a
//                        query or subquery on a point set.  (Note
//                        that when you run a query on a point
//                        universe, many QuerySolutionSets may be
//                        created, one for each PointSet.

import java.util.*;  // for vector
import java.io.*;    // for reading files
import java.awt.*;
import java.awt.image.ImageObserver;



class QuerySolutionSet<Object> extends LinkedList<Object>
{
    //this is a hashmap whose keys are edges to be drawn. 
    private HashMap edges;
    
    // this variable is no longer used, but it should probably go back
    // in somewhere.
    //
    // what's the most solutions we'll give back -- above this and we
    // print an error.  no one could reasonably want to look at a huge
    // number of results.
    public static final int MAX_SOLUTIONS = 5000;
   
    QuerySolutionSet()
    {
        super();
    }
    
    QuerySolutionSet(HashMap edgesToDraw)
    {
        super();
        edges = edgesToDraw;
    }
    
    public HashMap getEdgesToDraw() {
        return edges;
    }

    // return a copy of the original set with the dupes removed
    static QuerySolutionSet eliminateDuplicates(QuerySolutionSet solutions)
    {
	QuerySolutionSet<QuerySolution> setWithoutDupes = new QuerySolutionSet<QuerySolution>(solutions.getEdgesToDraw());

	// a hashset of hashsets.
	// 
	// the inner hashset contains a list of points that has
	// already appeared in this solution set.  a second solution
	// involving the same points will be excluded.
	HashSet<HashSet> usedSolutions = new HashSet<HashSet>();

	for (Iterator iter = solutions.iterator();iter.hasNext();)
	{
	    QuerySolution currentSolution = (QuerySolution)iter.next();

	    // if we want to keep duplicates or we haven't seen a
	    // solution involving these points...
	    if (!usedSolutions.contains(currentSolution.getPointList()))
	    {
		// add this solution to the list of de-duped
		// solutions, and make sure we don't keep it another
		// time after this one
		setWithoutDupes.add(currentSolution);
		usedSolutions.add(currentSolution.getPointList());
	    }
	}
	OutputWriter.println("removed "+(solutions.size()-setWithoutDupes.size())
			     +" duplicate solutions ("+setWithoutDupes.size()+" solutions remain)");

	return setWithoutDupes;
    }
    
    
    
    static boolean writeSVGSolutions(String description, String path, String gif, QuerySolutionSet solutions, PointSet ps, Writer fw) { 
        return writeSVGSolutions(description, path, gif, solutions, ps, fw, 1.0f, 50.0f);
    }
    
    static boolean writeSVGSolutions(String description, String path, String gif, QuerySolutionSet solutions, PointSet ps, Writer fw, float mult, float trans) {
        try {
            fw.write("<?xml version=\"1.0\" standalone=\"no\"?>\n");
            fw.write("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
            fw.write("<svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" style=\"width:100%; height:100%\">\n"); // width=\"100%\" height=\"100%\">\n");
            fw.write("<script type=\"text/ecmascript\"><![CDATA[ \n");
            fw.write("\t" + "var currID = 1; \n");
            fw.write("\t" + "var totinfo = \" of " + solutions.size() + "\"; \n");
      
            fw.write("\t" + "function seeNext(evt) { \n");
            fw.write("\t" + "\t" + "// Get Document \n");
            fw.write("\t" + "\t" + "var target = evt.getTarget(); \n");
            fw.write("\t" + "\t" + "var doc = target.getOwnerDocument(); \n");
            fw.write("\t" + "\t" + "// Make current invisible  \n");
            fw.write("\t" + "\t" + "var currVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "currVis.setAttribute(\"visibility\", \"hidden\"); \n");
            fw.write("\t" + "\t" + "currID++; \n");
            fw.write("\t" + "\t" + "// Make next visible \n");
            fw.write("\t" + "\t" + "var nxtVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "if (nxtVis == null) { \n");
            fw.write("\t" + "\t" + "\t" + "currID = 1; \n");
            fw.write("\t" + "\t" + "\t" + "nxtVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "} \n");            
            fw.write("\t" + "\t" + "nxtVis.setAttribute(\"visibility\", \"visible\"); \n");
            fw.write("\t" + "\t" + "//set the text \n");
            fw.write("\t" + "\t" + "var txtB = doc.getElementById(\"disp\"); \n");
            fw.write("\t" + "\t" + "txtB.getFirstChild().setData(currID + totinfo); \n");
            fw.write("\t" + "} \n");
            
            fw.write("\t" + "function seePrev(evt) { \n");
            fw.write("\t" + "\t" + "// Get Document \n");
            fw.write("\t" + "\t" + "var target = evt.getTarget(); \n");
            fw.write("\t" + "\t" + "var doc = target.getOwnerDocument(); \n");
            fw.write("\t" + "\t" + "// Make current invisible  \n");
            fw.write("\t" + "\t" + "var currVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "currVis.setAttribute(\"visibility\", \"hidden\"); \n");
            fw.write("\t" + "\t" + "currID--; \n");
            fw.write("\t" + "\t" + "// Make next visible \n");
            fw.write("\t" + "\t" + "var nxtVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "if (nxtVis == null) { \n");
            fw.write("\t" + "\t" + "\t" + "currID = " + solutions.size() + "; \n");
            fw.write("\t" + "\t" + "\t" + "nxtVis = doc.getElementById(currID); \n");
            fw.write("\t" + "\t" + "} \n");            
            fw.write("\t" + "\t" + "nxtVis.setAttribute(\"visibility\", \"visible\"); \n");
            fw.write("\t" + "\t" + "//set the text \n");
            fw.write("\t" + "\t" + "var txtB = doc.getElementById(\"disp\"); \n");
            fw.write("\t" + "\t" + "txtB.getFirstChild().setData(currID + totinfo); \n");
            fw.write("\t" + "} \n");

            
            fw.write("]]></script> \n");

            if(solutions.size()>0) {
                fw.write("<g onclick=\"seePrev(evt)\"> \n");
                fw.write("\t" + "<rect x=\"0\" y=\"0\" width=\"50\" height=\"20\" stroke=\"blue\" fill=\"white\" stroke-width=\"1\"/>  \n");
                fw.write("\t" + "<text id=\"prev\" x=\"10\" y=\"15\" opacity=\"1\">Prev</text> \n");
                fw.write("</g> \n");

                fw.write("<g> \n");
                fw.write("\t" + "<rect x=\"50\" y=\"0\" width=\"100\" height=\"20\" stroke=\"blue\" fill=\"white\" stroke-width=\"1\"/>  \n");
                fw.write("\t" + "<text id=\"disp\" x=\"60\" y=\"15\" opacity=\"1\">1 of " + solutions.size() + "</text> \n");
                fw.write("</g> \n");            

                fw.write("<g onclick=\"seeNext(evt)\"> \n");
                fw.write("\t" + "<rect x=\"150\" y=\"0\" width=\"50\" height=\"20\" stroke=\"blue\" fill=\"white\" stroke-width=\"1\"/>  \n");
                fw.write("\t" + "<text id=\"next\" x=\"160\" y=\"15\" opacity=\"1\">Next</text> \n");
                fw.write("</g> \n");
            }    
            else {
                fw.write("<g> \n");
                fw.write("\t" + "<rect x=\"50\" y=\"0\" width=\"100\" height=\"20\" stroke=\"blue\" fill=\"white\" stroke-width=\"1\"/>  \n");
                fw.write("\t" + "<text id=\"disp\" x=\"60\" y=\"15\" opacity=\"1\">0 of " + solutions.size() + "</text> \n");
                fw.write("</g> \n");                               
            }                            

            fw.write("\t" + "<text id=\"desc\" x=\"210\" y=\"15\" opacity=\"1\">" + description + "</text> \n");
            
            if(gif.length()>0) {

                //System.out.println(gif);
                ImageInfo ii = new ImageInfo();
                if(QueryBean.SERVER)
                    ii.setInput(new RandomAccessFile(new File(path+gif), "r"));
                else
                    ii.setInput(new RandomAccessFile(new File(gif), "r"));
                
                if(ii.check()) {   
                    fw.write("<g> \n");
                    fw.write("<image x=\"50\" y=\"50\" width=\"" + ii.getWidth() + "\" height=\"" + ii.getHeight() + "\" xlink:href=\"" + gif + "\"/>\n");
                    //fw.write("<image x=\"50\" y=\"50\" width=\"" + 800 + "\" height=\"" + 600 + "\" xlink:href=\"" + gif + "\"/>\n");
                    fw.write("</g> \n");
                }
            }
            
            fw.write(ps.getSVG(mult, trans) + "\n");
            
            int id = 0; 
            Iterator iter = solutions.iterator();
            while (iter.hasNext()) {
                QuerySolution qs = (QuerySolution)iter.next();
                fw.write(qs.getSVG(solutions.getEdgesToDraw(), mult, trans, ++id, (id == 1)) + "\n");
            }
            
            fw.write("</svg>");
            fw.flush();
            fw.close();
        }
        catch (IOException ioe) {
           OutputWriter.printerror("ShapeQueryExecuter: Error writing to BufferedWriter: " + ioe.getMessage() + ".\n");
           return false;
        }
        return true;
    }
}

